KEY_ATTRS = (
    'cache_control',
    'content_disposition',
    'content_encoding',
    'content_type',
    'content_language',
    'expires',
    'reduced_redundancy',
    'acl',
    'canned_acl',
    'encrypt',
    'redirect',
    'metadata',
)

KEY_HEADERS = (
    'cache_control',
    'content_disposition',
    'content_encoding',
    'content_type',
    'content_language',
    'expires',
)

BUCKET_ATTRS = (
    'bucket',
    'canned_acl',
    'cors',
    'key_config',
    'lifecycle',
    'region',
    'rsync',
    'website',
    #'acl',
    #'logging',
    #'notification',
    #'policy',
    #'request_payment',
    #'tagging',
    #'versioning',
)